package huffman;

import org.junit.Assert;
import org.junit.Test;

public class TestTreeEncodeAndDecode {
	
	@Test public void testTreeEncodeAndDecode0() {
		int[] frequencies = new int[] { 17, 16, 15, 10, 7 } ;
		char[] symbols = {'c', 'b', 'a', 'e', 'd' } ;
		char[] message = new char[] {'a','b','c','d','e','e','d','c','b','a'} ;
		
		encodeDecodeAndCheck(frequencies, symbols, message, 24);
	}
	
	@Test public void testTreeEncodeAndDecode1() {
		// This example is from the Wikipedia.
		// The string is "this is an example of a huffman tree"
		int[] frequencies = new int[]
		// 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15
		{  7, 4, 4, 3, 2, 2, 2, 2, 2, 2,  1,  1,  1, 1,  1,  1 } ;
		char[] symbols = new char[]
		// 0,  1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15
		{' ', 'a', 'e', 'f', 'h', 'i', 'n', 'm', 's', 't', 'p', 'r', 'l', 'o', 'u', 'x'} ;
		char[] message = new char[]
		{ 't', 'h', 'i', 's', ' ', 'i', 's', ' ', 'a', 'n',  ' ',
		  'e', 'x', 'a', 'm', 'p', 'l', 'e', ' ',           
		  'o', 'f', ' ', 'a', ' ', 'h', 'u', 'f', 'f', 'm', 'a', 'n',
		  ' ', 't', 'r', 'e', 'e' } ;
		
		encodeDecodeAndCheck(frequencies, symbols, message, 135);
	}
	
	@Test public void testTreeEncodeAndDecode3() {
		// Check that a length 1 frequency table is not a problem.
		// This is a very interesting case as the code point
		// can be coded using 0 bits!
		int[] frequencies = new int[]{ 10 } ;
		char[] symbols = new char[] {'A'} ;
		char[] message = new char[]{ 'A', 'A', 'A' } ;
		
		encodeDecodeAndCheck(frequencies, symbols, message, 0);
	}

	private void encodeDecodeAndCheck(int[] frequencies, char[] symbols, char[] message, int expectedLength) {
		Tree tree = new Tree( frequencies, symbols ) ;

		MockBooleanSequence seq = new MockBooleanSequence() ;
		TestUtilities.encodeMessage(tree, message, seq) ; 
		Assert.assertEquals( expectedLength, seq.size() ) ;
		
		char[] decodedMessage = TestUtilities.decodeMessage(tree, seq, message.length) ; 
		TestUtilities.compareArrays( message, decodedMessage ) ;
	}
}
